#include "../../../lib/std.mi"
#include "attribs.m"

Function reset();
Function createNotifier();
Function showNotifier(Int w);
Function onNext();

Function Int fillNextTrackInfo(String corneroverride);
Function Int fillCustomInfo(String customstring);

Function checkPref(int bypassfs);

Global Container containerNotifier;
Global Layout layoutNotifier;
Global Timer timerNotifier;
Global String strLastAutoTitle;
Global Int intScreenMargin;
Global String strPosition;

// ------------------------------------------------------------------------------
// init
// ------------------------------------------------------------------------------
System.onScriptLoaded() {
	initAttribs();
	intScreenMargin = 10;
	timerNotifier = new Timer;
}

// ------------------------------------------------------------------------------
// shutdown
// ------------------------------------------------------------------------------
System.onScriptUnloading() {
	delete timerNotifier;
}

// ------------------------------------------------------------------------------
// called by the system when the global hotkey for notification is pressed
// ------------------------------------------------------------------------------
System.onShowNotification() {
	if (checkPref(1)) return;
	createNotifier();
	String str;
	if (getStatus() == STATUS_PLAYING) str = "Playing";
	if (getStatus() == STATUS_PAUSED) str = "Playback Paused";
	if (getStatus() == STATUS_STOPPED) str = "Playback Stopped";
	showNotifier(fillNextTrackInfo(str));
	complete; // prevents other scripts from getting the message
	return 1; // tells anybody else that might watch the returned value that, yes, we implemented that
}

// ------------------------------------------------------------------------------
// called by the system when the title for the playing item changes, this could be the result of the player
// going to the next track, or of an update in the track title
// ------------------------------------------------------------------------------
System.onTitleChange(String newtitle) {
	if (strLastAutoTitle == newtitle) return;
	if (StrLeft(newtitle, 1) == "[") {
		if (StrLeft(newtitle, 7) == "[Buffer" ||
				StrLeft(newtitle, 4) == "[ICY") return;
	}
	strLastAutoTitle = newtitle;
	onNext();
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the next button
// ------------------------------------------------------------------------------
onNext() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo(""));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onPlay() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo("Playing"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button
// ------------------------------------------------------------------------------
System.onPause() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillCustomInfo("Playback Paused"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button again
// ------------------------------------------------------------------------------
System.onResume() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo("Resuming Playback"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onStop() {
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillCustomInfo("End of Playback"));
}

// ------------------------------------------------------------------------------
// checks if we should display anything
// ------------------------------------------------------------------------------
Int checkPref(int bypassfs) {
	if (!bypassfs && configAttribute_notifierDisableFullscreen.getData() == "1" && isVideoFullscreen()) return 1;
	if (configAttribute_notifierNever.getData() == "1") return 1;
	if (configAttribute_notifierMinimized.getData() == "1" && !isMinimized()) return 1;
	if (configAttribute_notifierWindowshade.getData() == "1") {
		if (isMinimized()) return 0;
		Container c = getContainer("main");
		if (!c) return 1;
		Layout l = c.getCurLayout();
		if (!l) return 1;
		if (l.getId() != "shade") return 1;
	}
	return 0;
}

// ------------------------------------------------------------------------------
// fade in/out completed
// ------------------------------------------------------------------------------
layoutNotifier.onTargetReached() {
	if (strPosition == "IN") {
		timerNotifier.setDelay(StringToInteger(configAttribute_notifierHoldtime.getData()));
		timerNotifier.start();
	} else if (strPosition == "OUT") {
		reset();	
	}
	/*
	int y = layoutNotifier.getTop();
	int x = layoutNotifier.getLeft();
	
	if (y < getViewportHeight() + getViewportTop() || x < getViewportWidth() + getViewportLeft()) {
		timerNotifier.setDelay(StringToInteger(configAttribute_notifierHoldtime.getData()));
		timerNotifier.start();
	} else {
		reset();
	}
	*/
}

// ------------------------------------------------------------------------------
// when notifier is clicked, bring back the app from minimized state if its minimized and focus it
// ------------------------------------------------------------------------------
layoutNotifier.onLeftButtonDown(int x, int y) {
	timerNotifier.stop();
	layoutNotifier.cancelTarget();
	reset();
	restoreApplication();
	activateApplication();
}

// ------------------------------------------------------------------------------
// close the notifier window, destroys the container automatically because it's dynamic
// ------------------------------------------------------------------------------
reset() {
	containerNotifier.close();
	containerNotifier = NULL;
	layoutNotifier = NULL;
}

// ------------------------------------------------------------------------------
createNotifier() {
	if (containerNotifier == NULL) {
		containerNotifier = newDynamicContainer("notifier");
		if (!containerNotifier) return; // reinstall duh!
//		if (isDesktopAlphaAvailable())
//			layoutNotifier = containerNotifier.getLayout("desktopalpha");
//		else
			layoutNotifier = containerNotifier.getLayout("normal");
		if (!layoutNotifier) return; // reinstall twice, man
	} else {
		layoutNotifier.cancelTarget();
		timerNotifier.stop();
	}
}

// ------------------------------------------------------------------------------
showNotifier(int w) {
	int intW = w + 45;
	int intH = System.stringToInteger(layoutNotifier.getXMLParam("h"));

	int intRightOnscreenX = getViewportWidth() + getViewportLeft() - intW - intScreenMargin;
	int intRightOffscreenX = getViewportWidth() + getViewportLeft();
	int intLeftOnScreenX = intScreenMargin;
	int intLeftOffscreenX = intW * -1;
	
	int intYHigh = getViewportHeight() + getViewportTop() - intH - intScreenMargin;
	int intYLow = getViewportHeight() + getViewportTop() + 25; //accomodate taskbar height :(
	
	int intSelectedStartX;
	int intSelectedStartY;
	int intSelectedFinalX;
	int intSelectedFinalY;
	
	if (configAttribute_notifierPositionRight.getData() == "1") {
		if (configAttribute_notifierSlideVertical.getData() == "1") {
		intSelectedStartX = intRightOnscreenX;
		intSelectedStartY = intYLow;
		intSelectedFinalX = intRightOnscreenX;
		intSelectedFinalY = intYHigh;
	} else {
		intSelectedStartX = intRightOffscreenX;
		intSelectedStartY = intYHigh;
		intSelectedFinalX = intRightOnscreenX;
		intSelectedFinalY = intYHigh;
	}
	} else {
		if (configAttribute_notifierSlideVertical.getData() == "1") {
		intSelectedStartX = intLeftOnScreenX;
		intSelectedStartY = intYLow;
		intSelectedFinalX = intLeftOnScreenX;
		intSelectedFinalY = intYHigh;
	} else {
		intSelectedStartX = intLeftOffscreenX;
		intSelectedStartY = intYHigh;
		intSelectedFinalX = intLeftOnScreenX;
		intSelectedFinalY = intYHigh;
	}
	}

	if (!layoutNotifier.isVisible()) {
		layoutNotifier.resize(intSelectedStartX, intSelectedStartY, intW, intH);
	}
	layoutNotifier.show();
	layoutNotifier.setAlpha(255);
	layoutNotifier.setTargetX(intSelectedFinalX);
	layoutNotifier.setTargetY(intSelectedFinalY);
	layoutNotifier.setTargetW(intW);
	layoutNotifier.setTargetH(intH);
	layoutNotifier.setTargetSpeed(StringToInteger(configAttribute_notifierSlideTime.getData()));
	layoutNotifier.gotoTarget();
	strPosition = "IN";
}

// ------------------------------------------------------------------------------
// hold time elapsed
// ------------------------------------------------------------------------------
timerNotifier.onTimer() {
	stop();
	
	int intW = w + 32;
	int intH = layoutNotifier.getHeight();
	
	int intRightOnscreenX = getViewportWidth() + getViewportLeft() - intW - intScreenMargin;
	int intRightOffscreenX = getViewportWidth() + getViewportLeft();
	int intLeftOnScreenX = intScreenMargin;
	int intLeftOffscreenX = intW * -1;
	
	int intYHigh = getViewportHeight() + getViewportTop() - intH - intScreenMargin;
	int intYLow = getViewportHeight() + getViewportTop() + 25; //accomodate taskbar height :(
	
	int intSelectedFinalX;
	int intSelectedFinalY;
	
	if (configAttribute_notifierPositionRight.getData() == "1") {
		if (configAttribute_notifierSlideVertical.getData() == "1") {
		intSelectedFinalX = intRightOnscreenX;
		intSelectedFinalY = intYLow;
	} else {
		intSelectedFinalX = intRightOffscreenX;
		intSelectedFinalY = intYHigh;
	}
	} else {
		if (configAttribute_notifierSlideVertical.getData() == "1") {
		intSelectedFinalX = intLeftOnScreenX;
		intSelectedFinalY = intYLow;
	} else {
		intSelectedFinalX = intLeftOffscreenX;
		intSelectedFinalY = intYHigh;
	}
	}
	
	layoutNotifier.setTargetX(intSelectedFinalX);
	layoutNotifier.setTargetY(intSelectedFinalY);
	layoutNotifier.setTargetSpeed(StringToInteger(configAttribute_notifierSlideTime.getData()));
	layoutNotifier.gotoTarget();
	strPosition = "OUT";
}

// ------------------------------------------------------------------------------
Int fillNextTrackInfo(String corneroverride) {
	Int maxv = 0;
	Int stream = 0;
	Group p = layoutNotifier;
	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	DebugString("got callback for " + getPlayItemString(), 0);
	if (StrLeft(getPlayItemString(), 7) == "http://") stream = 1;

	if (endofplayback) endofplayback.hide();

	if (plentry) { plentry.setText(integerToString(getPlaylistIndex()+1)+"/"+integerToString(getPlaylistLength())); plentry.show(); }
	if (nexttrack) {
		if (corneroverride == "") {
			if (!stream) {
				if (!isVideo())
					nexttrack.setText("New Track");
				else
					nexttrack.setText("New Video");
			}
			else nexttrack.setText("On Air");
		} else nexttrack.setText(corneroverride);
		nexttrack.show();
	}
	if (_title) {
		String str;
		if (!stream) {
			_title.setXmlParam("ticker", "0");
			_title.setXmlParam("display", "");
			str = getPlayitemMetaDataString("title"); 
			if (str == "") str = getPlayitemDisplayTitle();
			String l = getPlayItemMetaDataString("length");
			if (l != "") {
				str += " (" + integerToTime(stringtointeger(l)) + ")";
			}
			_title.setText(str);
		} else {
			_title.setXmlParam("ticker", "1");
			_title.setXmlParam("display", "songtitle");
			_title.setText("");
		}
		_title.show(); 
	}
	if (artist) { 
		if (!stream) {
			if (isVideo())
				artist.setText(""); 
			else
				artist.setText(getPlayitemMetaDataString("artist")); 
		} else {
			if (isVideo())
				artist.setText("Internet TV"); 
			else
				artist.setText("Internet Radio"); 
		}
		artist.show(); 
	}
	if (album) { 
		String str;
		if (!stream && !isVideo()) {
			album.setXmlParam("display", "");
			str = getPlayitemMetaDataString("album");
			String l = getPlayitemMetaDataString("track");
			if (l != "" && l != "-1") str += " (Track " + l + ")";
			album.setText(str); 
		} else {
			album.setText("");
			album.setXmlParam("display", "songinfo");
		}
		album.show(); 
	}
	maxv = artist.getAutoWidth();
	if (maxv < album.getAutoWidth()) maxv = album.getAutoWidth();
	if (maxv < _title.getAutoWidth()) maxv = _title.getAutoWidth();
	if (maxv < 128) maxv = 128;
	if (maxv > getViewportWidth()/4) maxv = getViewportWidth()/4;
	
	return maxv;
}

// ------------------------------------------------------------------------------
Int fillCustomInfo(String customtext) {
	Group p = layoutNotifier;
	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	if (plentry) { plentry.hide(); }
	if (nexttrack) nexttrack.hide();
	if (_title) { _title.hide(); }
	if (artist) { artist.hide(); }
	if (album) { album.hide(); }

	if (endofplayback) {
		endofplayback.setText(customtext);
		int aw = endofplayback.getAutoWidth();
		endofplayback.show();
		if (aw > 128)
			return aw;
	}
	return 128;
}

